/*
 *  gui.h: User interface
 *
 *  Copyright (C) 2010- John Coppens (john@jcoppens.com)        
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 3 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *  
 *  You should have received a copy of the GNU Library General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <gtk/gtk.h>

enum {	DSP_DECIMAL,
	DSP_HEXADECIMAL,
	DSP_OCTAL,
	DSP_BINARY,
	DSP_NR_MODES
};

enum {	CONV_FROM,
	CONV_TO,
	CONV_NR_COLS
};

enum {	VARIABLES_NAME,
	VARIABLES_VALUE,
	VARIABLES_COMMENT,
	VARIABLES_NR_COLS
};

#define STDOUT_BFF_SIZE	500
#define STDERR_BFF_SIZE 500

#define	CONST_PI	"\u03c0"
#define CONST_E		"e"
#define CONST_EULER	"\u03b3"
#define CONST_E_GRAVITY	"g"
#define CONST_COULOMB	"Cc"
#define CONST_UNIVERSAL	"Z0"
#define CONST_BOHR_MAG	"muB"
#define CONST_NUCL_MAG	"muN"
#define CONST_CONDUCT_Q	"G0"
#define CONST_EL_CHARGE	"ec"
#define CONST_JOSEPHSON	"Kj"
#define CONST_KLITZING	"Rk"
#define CONST_Z_VACUUM	"Z0"
#define CONST_EPSILON0	"\u03b50"
#define CONST_MU0	"\u03bc0"
#define CONST_GRAVITY	"G"
#define CONST_PLANCK	"h"
#define CONST_LIGHTSP	"c"

#define CONST_MALPHA	"Malpha"
#define CONST_BOHR_RAD	"a0"
#define CONST_DEUT_MASS	"Md"
#define CONST_ELEC_MASS	"Me"
#define CONST_ELEC_RAD	"re"
#define CONST_FERMI	"Gf"
#define	CONST_FINE_STR	"alpha"
#define CONST_HARTREE	"eh"
#define	CONST_HELION	"Mh"
#define CONST_MUON	"Mmu"
#define CONST_NEUTRON	"Mn"
#define CONST_PROTON	"Mp"
#define CONST_RYDBERG	"Rinf"
#define CONST_TAU_MASS	"Mt"

#define CONST_ATOM_MASS	"u"
#define CONST_AVOGADRO	"Na"
#define CONST_BOLTZMANN	"k"
#define	CONST_FARADAY	"F"
#define	CONST_RAD_1	"c1"
#define CONST_LOSCHMIDT	"n0"
#define CONST_MOLAR_GAS	"R"
#define CONST_MOLAR_VOL	"Vm"
#define CONST_RAD_2	"c2"
#define CONST_SIGMA	"\u03c3"
#define CONST_WIEN_DISPL "b"

#define CONST_MAGFLUX_Q	"\u03d50"
#define CONST_CATALAN	"K"
#define CONST_RAND	"random"
#define CONST_IRAND	"irandom"

#define MAX_NR_DECIMALS	145

//////	Extras popup
//	ExtrasPopup - functions
void		sine1_activate(void);
void		cosine1_activate(void);
void		tangent1_activate(void);
void		secant1_activate(void);
void		cosecant1_activate(void);
void		cotangent1_activate(void);

void		arcsine1_activate(void);
void		arccosine1_activate(void);
void		arctangent1_activate(void);
void		arcsecant1_activate(void);
void		arccosecant1_activate(void);
void		arccotangent1_activate(void);

void		hyperbolic_sine1_activate(void);
void		hyperbolic_cosine1_activate(void);
void		hyperbolic_tangent1_activate(void);
void		hyperbolic_secant1_activate(void);
void		hyperbolic_cosecant1_activate(void);
void		hyperbolic_cotangent1_activate(void);

void		inverted_hyperbolic_sine1_activate(void);
void		inverted_hyperbolic_cosine1_activate(void);
void		inverted_hyperbolic_tangent1_activate(void);
void		inverted_hyperbolic_secant1_activate(void);
void		inverted_hyperbolic_cosecant1_activate(void);
void		inverted_hyperbolic_cotangent1_activate(void);

void		natural_logarithm1_activate(void);
void		base_10_logarithm1_activate(void);
void		base_2_logarithm1_activate(void);
void		gamma_function1_activate(void);
void		log_gamma_function1_activate(void);
void		zeta_function1_activate(void);
void		sampling_function1_activate(void);
void		nearest_integer1_activate(void);
void		absolute_value1_activate(void);
void		ones_complement1_activate(void);
void		exponential_integral1_activate(void);
void		exponent1_activate(void);
void		factorial1_activate(void);
void		square_root1_activate(void);
void		cube_root1_activate(void);
void		next_lower_integer1_activate(void);
void		next_higher_integer1_activate(void);
void		random_value1_activate(void);
void		random_integer1_activate(void);

void		conversions1_activate(void);

//	ExtrasPopup - Constants
void		pi1_activate(void);
void		e1_activate(void);
void		eulers_constant1_activate(void);
void		catalan1_activate(void);
void		gravity1_activate(void);
void		coulomb1_activate(void);

void		impedancevacuum1_activate(void);
void		epsilon1_activate(void);
void		mu1_activate(void);
void		gravitational_constant1_activate(void);
void		planck_constant1_activate(void);
void		speed_of_light1_activate(void);

void		alpha_particle_mass1_activate(void);
void		bohr_radius1_activate(void);
void		deuteron_mass1_activate(void);
void		electron_mass1_activate(void);
void		electron_radius1_activate(void);
void		fermi_coupling_constant1_activate(void);
void		fine_structure_constant1_activate(void);
void		hartree_energy1_activate(void);
void		helion_mass1_activate(void);
void		muon_mass1_activate(void);
void		neutron_mass1_activate(void);
void		proton_mass1_activate(void);
void		rydberg_constant1_activate(void);
void		tau_mass1_activate(void);

void		atomic_mass_constant1_activate(void);
void		avogadros_constant1_activate(void);
void		boltzmann_constant1_activate(void);
void		faraday_constant1_activate(void);
void		first_radiation_constant1_activate(void);
void		loschmidt_constant1_activate(void);
void		molar_mass_constant1_activate(void);
void		molar_volumn_ideal_gas1_activate(void);
void		second_radiation_constant1_activate(void);
void		sigma1_activate(void);
void		wien_displacement_law1_activate(void);

void		bohr_magneton1_activate(void);
void		nuclear_magneton1_activate(void);
void		conductance_quantum1_activate(void);
void		elementary_charge1_activate(void);
void		josephson_constant1_activate(void);
void		von_klitzing_constant1_activate(void);
void		magnetic_flux_quantum1_activate(void);
void		physio_chemical_constants1_activate(void);
void		a_random_value1_activate(void);
void		a_random_integer1_activate(void);

//	ExtrasPopup - others
void		inspector1_activate(void);
void		variables2_activate(void);
void		preferences2_activate(void);
void		quit2_activate(void);

//////	ResultPopup
void		save_to_variable3_activate(void);
void		conversions3_activate(void);

//////  PrefWindow
void		pref_cancel_btn_clicked(void);
void		pref_ok_btn_clicked(void);
void		PrefWindow_realize(void);
gboolean	PrefWindow_delete_event(void);


//////  MainWindow
//	MainWindow - menubar
void		clear1_activate(void);
void		history1_activate(void);
void		variables1_activate(void);
void		preferences1_activate(void);
void		quit1_activate(void);
void		conversions2_activate(void);
void		decimal1_activate(void);
void		hexadecimal1_activate(void);
void		octal1_activate(void);
void		binary1_activate(void);
void		recalculate1_activate(void);
void		about1_activate(void);

//	MainWindow - events
gboolean	MainWindow_delete_event(void);
void		MainWindow_realize(void);
void		main_btn_clicked(int key);
void		main_mode_btn_clicked(void);
gboolean	main_input_txtview_button_press_event(GdkEventButton *event);
gboolean	main_result_txtview_button_press_event(GdkEventButton *event);
gboolean	main_input_txtview_key_press_event(GdkEventKey *event);
void		save_to_variable2_activate(void);
gboolean	main_eventbox_button_press_event(GtkWidget *widget, GdkEventButton *event);

//	MainWindow - utility functions
void		error_dialog(char *ttl, char *msg);


//////  ConversionsWindow
void		conv_family_cbbox_changed(void);
void		conv_from_treeview_cursor_changed(void);
void		conv_to_treeview_cursor_changed(void);
void		conv_cancel_btn_clicked(void);
void		conv_ok_btn_clicked(void);
gboolean	ConversionsWindow_delete_event(void);
void		ConversionsWindow_realize(void);
void		ConversionsWindow_show(void);
void		conv_from_treeview_realize(void);
    
//////  VariablesWindow
gboolean	VariablesWindow_delete_event(void);
void		VariablesWindow_realize(void);
void		var_treeview_cursor_changed(void);
void		var_add_btn_clicked(void);
void		var_remove_btn_clicked(void);
void		var_clear_btn_clicked(void);
void		var_cancel_btn_clicked(void);
void		var_ok_btn_clicked(void);

//////  InspectorWindow
gboolean	InspectorWindow_delete_event(void);
void		InspectorWindow_realize(void);
void		insp_which_cbbox_changed(void);
void		insp_treeview_cursor_changed(void);
void		insp_ok_btn_clicked(void);


void		clear_form_entry(void);
void		set_display_mode(int);
void		execute_formula(void);

//////	AddVariableWindow

void		addvar_cancel_btn_clicked(void);
void		addvar_ok_btn_clicked(void);
void		AddVariableWindow_realize(void);
gboolean	AddVariableWindow_delete_event(void);
